{
  Solutions to the Advent Of Code.
  Copyright (C) 2023  Stefan Müller

  This program is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit UCosmicExpansionTestCases;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testregistry, USolver, UBaseTestCases, UCosmicExpansion;

type

  { TCosmicExpansionFullDataTestCase }

  TCosmicExpansionFullDataTestCase = class(TEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart1;
    procedure TestPart2;
  end;

  { TCosmicExpansionExampleTestCase }

  TCosmicExpansionExampleTestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart1;
  end;

  { TFactor10CosmicExpansion }

  TFactor10CosmicExpansion = class(TCosmicExpansion)
    function GetExpansionFactor: Integer; override;
  end;

  { TCosmicExpansionExampleFactor10TestCase }

  TCosmicExpansionExampleFactor10TestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart2;
  end;

  { TFactor100CosmicExpansion }

  TFactor100CosmicExpansion = class(TCosmicExpansion)
    function GetExpansionFactor: Integer; override;
  end;

  { TCosmicExpansionExampleFactor100TestCase }

  TCosmicExpansionExampleFactor100TestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart2;
  end;

implementation

{ TCosmicExpansionFullDataTestCase }

function TCosmicExpansionFullDataTestCase.CreateSolver: ISolver;
begin
  Result := TCosmicExpansion.Create;
end;

procedure TCosmicExpansionFullDataTestCase.TestPart1;
begin
  AssertEquals(9686930, FSolver.GetResultPart1);
end;

procedure TCosmicExpansionFullDataTestCase.TestPart2;
begin
  AssertEquals(630728425490, FSolver.GetResultPart2);
end;

{ TCosmicExpansionExampleTestCase }

function TCosmicExpansionExampleTestCase.CreateSolver: ISolver;
begin
  Result := TCosmicExpansion.Create;
end;

procedure TCosmicExpansionExampleTestCase.TestPart1;
begin
  AssertEquals(374, FSolver.GetResultPart1);
end;

{ TFactor10CosmicExpansion }

function TFactor10CosmicExpansion.GetExpansionFactor: Integer;
begin
  Result := 9;
end;

{ TCosmicExpansionExampleFactor10TestCase }

function TCosmicExpansionExampleFactor10TestCase.CreateSolver: ISolver;
begin
  Result := TFactor10CosmicExpansion.Create;
end;

procedure TCosmicExpansionExampleFactor10TestCase.TestPart2;
begin
  AssertEquals(1030, FSolver.GetResultPart2);
end;

{ TFactor100CosmicExpansion }

function TFactor100CosmicExpansion.GetExpansionFactor: Integer;
begin
  Result := 99;
end;

{ TCosmicExpansionExampleFactor100TestCase }

function TCosmicExpansionExampleFactor100TestCase.CreateSolver: ISolver;
begin
  Result := TFactor100CosmicExpansion.Create;
end;

procedure TCosmicExpansionExampleFactor100TestCase.TestPart2;
begin
  AssertEquals(8410, FSolver.GetResultPart2);
end;

initialization

  RegisterTest(TCosmicExpansionFullDataTestCase);
  RegisterTest(TCosmicExpansionExampleTestCase);
  RegisterTest(TCosmicExpansionExampleFactor10TestCase);
  RegisterTest(TCosmicExpansionExampleFactor100TestCase);
end.

