{
  Solutions to the Advent Of Code.
  Copyright (C) 2023  Stefan Müller

  This program is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation, either version 3 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program.  If not, see <http://www.gnu.org/licenses/>.
}

unit UHauntedWastelandTestCases;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testregistry, USolver, UBaseTestCases, UHauntedWasteland;

type

  { THauntedWastelandFullDataTestCase }

  THauntedWastelandFullDataTestCase = class(TEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart1;
    procedure TestPart2;
  end;

  { THauntedWastelandExampleTestCase }

  THauntedWastelandExampleTestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart1;
  end;

  { TExample2HauntedWasteland }

  TExample2HauntedWasteland = class(THauntedWasteland)
    function GetDataFileName: string; override;
  end;

  { THauntedWastelandExample2TestCase }

  THauntedWastelandExample2TestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart1;
  end;

  { TExample3HauntedWasteland }

  TExample3HauntedWasteland = class(THauntedWasteland)
    function GetDataFileName: string; override;
  end;

  { THauntedWastelandExample3TestCase }

  THauntedWastelandExample3TestCase = class(TExampleEngineBaseTest)
  protected
    function CreateSolver: ISolver; override;
  published
    procedure TestPart2;
  end;

implementation

{ THauntedWastelandFullDataTestCase }

function THauntedWastelandFullDataTestCase.CreateSolver: ISolver;
begin
  Result := THauntedWasteland.Create;
end;

procedure THauntedWastelandFullDataTestCase.TestPart1;
begin
  AssertEquals(14257, FSolver.GetResultPart1);
end;

procedure THauntedWastelandFullDataTestCase.TestPart2;
begin
  AssertEquals(16187743689077, FSolver.GetResultPart2);
end;

{ THauntedWastelandExampleTestCase }

function THauntedWastelandExampleTestCase.CreateSolver: ISolver;
begin
  Result := THauntedWasteland.Create;
end;

procedure THauntedWastelandExampleTestCase.TestPart1;
begin
  AssertEquals(2, FSolver.GetResultPart1);
end;

{ TExample2HauntedWasteland }

function TExample2HauntedWasteland.GetDataFileName: string;
begin
  Result := 'haunted_wasteland2.txt';
end;

{ THauntedWastelandExample2TestCase }

function THauntedWastelandExample2TestCase.CreateSolver: ISolver;
begin
  Result := TExample2HauntedWasteland.Create;
end;

procedure THauntedWastelandExample2TestCase.TestPart1;
begin
  AssertEquals(6, FSolver.GetResultPart1);
end;

{ TExample3HauntedWasteland }

function TExample3HauntedWasteland.GetDataFileName: string;
begin
  Result := 'haunted_wasteland3.txt';
end;

{ THauntedWastelandExample3TestCase }

function THauntedWastelandExample3TestCase.CreateSolver: ISolver;
begin
  Result := TExample3HauntedWasteland.Create;
end;

procedure THauntedWastelandExample3TestCase.TestPart2;
begin
  AssertEquals(6, FSolver.GetResultPart2);
end;

initialization

  RegisterTest(THauntedWastelandFullDataTestCase);
  RegisterTest(THauntedWastelandExampleTestCase);
  RegisterTest(THauntedWastelandExample2TestCase);
  RegisterTest(THauntedWastelandExample3TestCase);
end.
